@@ -81,6 +81,7 @@ typedef enum {
 	PANEL_OBJECT_BONOBO,
 	PANEL_OBJECT_ACTION,
 	PANEL_OBJECT_MENU_BAR,
+	PANEL_OBJECT_SEPARATOR,
 	/* The following two are for backwards compatibility with 2.0.x */
 	PANEL_OBJECT_LOGOUT,
 	PANEL_OBJECT_LOCK
@@ -95,6 +96,7 @@ typedef enum {
         PANEL_ACTION_SCREENSHOT,
 	PANEL_ACTION_FORCE_QUIT,
 	PANEL_ACTION_CONNECT_SERVER,
+        PANEL_ACTION_SHUTDOWN,
         PANEL_ACTION_LAST
 } PanelActionButtonType;
 
